__author__ = 'Erdene-Ochir Tuguldur'

import time
import importlib

import synthDriverHandler
from speech.commands import LangChangeCommand


class NVDASynthesizer(object):

    def __init__(self, player, cancel_flag):
        self._player = player
        self._cancel_flag = cancel_flag
        self._synthesizer = None
        self._ibmeci = None

    def set_synthesizer(self, synthesizer_name):
        self.terminate()
        if synthesizer_name != 'none':
            self._synthesizer = synthDriverHandler._getSynthDriver(synthesizer_name)()
            self._synthesizer.initSettings()

            if synthesizer_name == 'ibmeci':
                self._ibmeci = importlib.import_module("synthDrivers.ibmeci", package="synthDrivers")._ibmeci

    def terminate(self):
        self._ibmeci = None
        if self._synthesizer is not None:
            self._synthesizer.terminate()
            self._synthesizer = None

    def synthesize(self, lang, text, log_func=None):
        # only play after the Mongolian TTS is finished
        self._player.sync()

        if self._synthesizer is None or self._cancel_flag.is_set():
            return

        if self._synthesizer.name == 'ibmeci':
            onDoneSpeakingOrigFunc = self._ibmeci.onDoneSpeaking
            _onDoneSpeaking = False

            def _onDoneSpeakingFunc():
                nonlocal _onDoneSpeaking
                _onDoneSpeaking = True

            self._ibmeci.onDoneSpeaking = _onDoneSpeakingFunc
            self._synthesizer.speak([LangChangeCommand(lang), text])

            while not _onDoneSpeaking:
                if self._cancel_flag.is_set():
                    self._synthesizer.cancel()
                    break
                time.sleep(0.01)
            self._ibmeci.onDoneSpeaking = onDoneSpeakingOrigFunc
            self._ibmeci.player.stop() if self._cancel_flag.is_set() else self._ibmeci.player.sync()

        elif self._synthesizer.name == 'oneCore':
            self._synthesizer.speak([LangChangeCommand(lang), text])

            while self._synthesizer._isProcessing:
                if self._cancel_flag.is_set():
                    self._synthesizer.cancel()
                    break
                time.sleep(0.01)
            self._synthesizer._player.stop() if self._cancel_flag.is_set() else self._synthesizer._player.sync()

        elif self._synthesizer.name == 'vocalizer_expressive2':
            self._synthesizer.speak([LangChangeCommand(lang), text])

            while self._synthesizer._bgQueue.qsize() > 0:
                if self._cancel_flag.is_set():
                    self._synthesizer.cancel()
                    break
                time.sleep(0.01)
            self._synthesizer._player.stop() if self._cancel_flag.is_set() else self._synthesizer._player.sync()

        elif self._synthesizer.name == 'espeak':
            self._synthesizer.speak([LangChangeCommand(lang), text])
